# CITIGOTCHI
#
# Based on Austin Z. Henley's original Java exercise in COSC365 hw5&6,
# at University of Tennessee at Knoxville.
# https://austinhenley.com
#
#
# Translated into Python by Nela Brockington, 20th November 2021
# for City of London School for Girls, London, U.K.
n#
####### Comp Sci A Level, OOP ######
# Python code to set up tamagotchi game of digital pets
#
# This is a text-based game that will repeatedly ask the player for a
# command. Each command has a different effect on the pet.
#
# Task 1A. Give the Citigotchi class a constructor. Use "dunder"/name
# mangling to make all the attributes inaccessible except through
# getters. Make getters and setters for all the attributes. Update all
# the methods to use the getters and setters.
#
# Task 1B. Make a class that inherits from Action. Add it to the game
# so that the new action can be selected by the player. Use the two
# existing actions as examples. Make it do something interesting.
#
# Task 1C. Make a class that inherits from the Citigotchi class to
# create your own species with a preset value for the species
# attribute. Make it so that the main() function, uses your class
# instead of Citigotchi.
#
#
# Task 2A. Make it so that you can have multiple pets. You will need
# to support: (a) obtaining a new pet and naming it, (b) switching
# between pets such that the actions menu applies the currently
# selected one, and (c) release a pet so that you no longer own
# it. All three of these should be listed in the actions
# menu. Obtaining a new pet should allow either choosing its species
# or be given a random one. Switching pets should ask which pet to
# switch to and the player has to enter that pet's name. You should
# have at least three different species of possible pets.
#
# Task 2B. Add an action that displays the status of the current pet
# (e.g., happiness, hunger, species)
#
# Task 2C. Make the menu system have actions that are displayed only
# when certain conditions are met. You will use this to create and
# show at least one species-specific action per species. For example,
# if you have a species of pet "Dragon", it may have a possible action
# "FLY", but no other species should display that on the menu. NOTE:
# This should not be hard-coded. It should be dynamic such that one
# could add additional species with different species-specific actions
# with little to no changes to other code, like the menu system. An IF
# statement for each species is not the way to do it.
#
# Optional extension: Add text art to the game to visualise the pets.
# 


class Citigotchi():
  # There should be a class constructor allowing
  # object attributes to be set by arguments.

  # Attributes should be "private" and have getters and setters.
  species = 'generic'
  happiness = 1.0
  hunger = 1.0



class Action():
  def getCommand( self ):
    pass

  def getMenuMessage( self ):
    pass

  def doAction( self , pet ):
    pass

  
class Eat( Action ):
  @staticmethod #decorator
  def getCommand():
    return 'EAT'

  @staticmethod
  def getMenuMessage():
    return 'Feed your pet.'

  @staticmethod
  def doAction( pet ):
    pet.hunger -= 0.25
    if pet.hunger < 0.5:
      pet.happiness += 0.25
      return 'Your pet is satisfied!'

    pet.happiness -= 0.1
    return 'Your pet is still hungry!'


class Exercise( Action ):
  @staticmethod #decorator
  def getCommand():
    return 'EXERCISE'

  @staticmethod
  def getMenuMessage():
    return 'Exercise with your pet.'

  @staticmethod
  def doAction( pet ):
    pet.hunger += 0.4;
    if pet.hunger < 0.5:
      pet.happiness += 0.2
      return 'Your pet had a good workout!.'
    
    pet.happiness -= 0.25;
    return 'Your pet is exhausted and hungry!'    



# Run the game:
def main():
  
  pet = Citigotchi() # Should be pet1 = YourClass(...), i.e., subclass
  pet.species = "dolphin" # This should be in the constructor
  pet.hunger = 0.2 # This should be in the constructor
  pet.happiness = 0.5 # This should be in the constructor

  actions = []
  actions.append( Eat() )
  actions.append( Exercise() )
  # Add your own action here

  print( '########################\n' )
  print( 'Welcome to Citigotchi. Your pet is a {}.\n'.format( pet.species ) )

  choice = ''

  while choice != 'QUIT':

    # Print the options: 
    print( 'Enter one of the following commands:\n' )    
    for action in actions:
      print( '{} --- {}'.format( action.getCommand() , action.getMenuMessage() ))
    print( 'QUIT --- Quit the game.' )

    # Perform specified action:
    choice = input(">> ").upper()
    didAction = False
    for action in actions: 
      if choice == action.getCommand():
        print( action.doAction( pet ) + '\n')
        didAction = True
        break

    if not didAction and choice != 'QUIT':
      print( 'Invalid action. Try again.\n' )

  print( '\nThanks for playing Citigotchi! See you next time!\n' )



main()

  
