# CITIGOTCHI: ANSWERS to TASK 1 A, B, C
#
# Based on Austin Z. Henley's original Java exercise in COSC365 hw5&6,                   
# at University of Tennessee at Knoxville.
# https://austinhenley.com
#
# Translated to Python, and Python answers written, by Nela
# Brockington, 20th November 2021 for City of London School for Girls,
# UK.
#
#
####### Comp Sci A Level, OOP ######
# Python code to set up tamagotchi game of digital pets
#
# This is a text-based game that will repeatedly ask the player for a
# command. Each command has a different effect on the pet.
#
#
# Task 1A. Give the Citigotchi class a constructor. Use "dunder"/name
# mangling to make all the attributes inaccessible except through
# getters. Make getters and setters for all the attributes. Update all
# the methods to use the getters and setters.
#
# Task 1B. Make a class that inherits from Action. Add it to the game so
# that the new action can be selected by the player. Use the two
# existing actions as examples. Make it do something interesting.
#
# Task 1C. Make a class that inherits from the Citigotchi class to
# create your own species with a preset value for the species
# attribute. Make it so that the main() function, uses your class
# instead of Citigotchi.
#


class Citigotchi():
  # Code edit: constructor for Citigotchi class
  # Code edit: dunder to indicate attributes are "private"
  def __init__( self , species , happiness , hunger ):
    self.__species = species
    self.__happiness = happiness
    self.__hunger = hunger

  # Code edit: getters and setters for attributes
  def getSpecies( self ):
    return self.__species

  def getHappiness( self ):
    return self.__happiness

  def getHunger( self ):
    return self.__hunger

  def setSpecies( self , s ):
    self.__species = s

  def setHappiness( self , h ):
    self.__happiness = h

  def setHunger( self , h ):
    self.__hunger = h

# Code edit: new subclass DOLPHIN that inherits from Citigotchi
class Dolphin( Citigotchi ):
  def __init__( self , happiness , hunger ):
    # Code edit: Using a preset value for the SPECIES attribute
    super( Dolphin , self ).__init__( 'dolphin' , happiness , hunger )



class Action():
  def getCommand( self ):
    pass

  def getMenuMessage( self ):
    pass

  def doAction( self , pet ):
    pass


class Eat( Action ):
  @staticmethod #decorator
  def getCommand():
    return 'EAT'

  @staticmethod
  def getMenuMessage():
    return 'Feed your pet.'

  @staticmethod
  def doAction( pet ):
    # Code edit: update method to use getters and setters    
    pet.setHunger( pet.getHunger() - 0.25 )
    if pet.getHunger() < 0.5:
      pet.setHappiness( pet.getHappiness() + 0.25 )
      return 'Your pet is satisfied!'

    pet.setHappiness( pet.getHappiness() - 0.1 )
    return 'Your pet is still hungry!'


class Exercise( Action ):
  @staticmethod #decorator
  def getCommand():
    return 'EXERCISE'

  @staticmethod
  def getMenuMessage():
    return 'Exercise with your pet.'

  @staticmethod
  def doAction( pet ):
    # Code edit: update method to use getters and setters    
    pet.setHunger( pet.getHunger() + 0.4 )
    if pet.getHunger() < 0.5:
      pet.setHappiness( pet.getHappiness() + 0.2 )
      return 'Your pet had a good workout!'
    
    pet.setHappiness( pet.getHappiness() - 0.25 )
    return 'Your pet is exhausted and hungry!'    

# Code edit: New subclass SLEEP that inherits from Action
class Sleep( Action ):
  @staticmethod # decorator                                                               
  def getCommand():
    return 'SLEEP'

  @staticmethod
  def getMenuMessage():
    return 'Let your pet have a nap.'

  @staticmethod
  def doAction( pet ):
    pet.setHappiness( pet.getHappiness() + 0.15 )
    if pet.getHunger() < 0.5:
      return 'Your pet is now rested and would like to exercise!'

    return 'Your pet is now rested and would like a snack!'




# Run the game:
def main():

  # Code edit: pet is instantiated from the new subclass DOLPHIN
  # Code edit: Attribute values are passed to a constructor 
  pet = Dolphin( 0.2 , 0.5 )

  actions = []
  actions.append( Eat() )
  actions.append( Exercise() )
  # Code edit: New Action SLEEP added to the game
  actions.append( Sleep() )
  

  print( '########################\n' )
  print( 'Welcome to Citigotchi. Your pet is a {}.\n'.format( pet.getSpecies() ) )

  choice = ''

  while choice != 'QUIT':

    # Print the options: 
    print( 'Enter one of the following commands:\n' )    
    for action in actions:
      print( '{} --- {}'.format( action.getCommand() , action.getMenuMessage() ))
    print( 'QUIT --- Quit the game.' )

    # Perform specified action:
    choice = input(">> ").upper()
    didAction = False
    for action in actions: 
      if choice == action.getCommand():
        print( action.doAction( pet ) + '\n')
        didAction = True
        break

    if not didAction and choice != 'QUIT':
      print( 'Invalid actipon. Try again.\n' )

  print( '\nThanks for playing Citigotchi! See you next time!\n' )


main()

  
